<?php
/**
 * @file
 * Drop Jobs Application pages.
 */

/**
 * Application view callback.
 * Displays an application entity.
 *
 * @see drop_jobs_application_menu()
 */
function drop_jobs_application_view($application) {
  drupal_set_title(entity_label('drop_jobs_application', $application));
  return entity_view('drop_jobs_application', array(entity_id('drop_jobs_application', $application) => $application), 'full');
}

/**
 * Form callback.
 *
 * Shows a confirmation form for applying to a job.
 *
 * @see drop_jobs_application_menu()
 */
function drop_jobs_application_form_apply($form, $form_state, $nid, $type = NULL) {
  // Redirect anonymous users to registration page.
  // This is necessary because we deliberately let anonymous users see the
  // "apply to job" link but sneakily redirect to user/register.
  if (user_is_anonymous()) {
    drupal_goto('user/register', array('query' => array('destination' => 'node/' . $nid . '/apply')));
    exit;
  }

  // Load allowed application types for this job.
  // This is more of a sanity check than anything else.
  $allowed = drop_jobs_application_allowed($nid);
  if (empty($allowed)) {
    drupal_access_denied();
    exit;
  }

  $node = node_load($nid);

  // Build application form.
  $form = array();

  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $nid,
  );

  // Load application types & filter according to allowed types.
  $options = array();
  foreach (drop_jobs_application_get_types() as $type_name => $object) {
    if (in_array($type_name, $allowed)) {
      $options[$type_name] = $object->label;
    }
  }

  // Sanity check.
  if (empty($options)) {
    drupal_access_denied();
    exit;
  }

  // Make sure we have a valid type.
  if (isset($type) && array_key_exists($type, $options)) {
    $application_type = $type;
  }
  elseif (count($options) === 1) {
    $options_temp = array_keys($options);
    $application_type = reset($options_temp);
  }

  // If we only have one application type, display the type-specific application
  // form. If not, display a form to select types.
  if (isset($application_type)) {
    $form['type'] = array(
      '#type' => 'hidden',
      '#value' => $application_type,
    );

    if (variable_get_value('drop_jobs_terms_path')) {
      $form['terms'] = array(
        '#type' => 'checkbox',
        '#title' => t('Terms and Conditions'),
        '#description' => t('Check the above box to confirm you read, understood and agree with our <a href="@terms">Terms and Conditions</a>', array('@terms' => drop_jobs_core_get_terms_link())),
        '#default_value' => FALSE,
        '#required' => TRUE,
        '#weight' => 99,
      );
    }

    $application = entity_create('drop_jobs_application', array('type' => $application_type));
    field_attach_form('drop_jobs_application', $application, $form, $form_state);

    $form['#validate'][] = 'drop_jobs_application_form_apply_validate_apply';
    $form['#submit'][] = 'drop_jobs_application_form_apply_submit_apply';

    return confirm_form($form, t('Apply to job %title?', array('%title' => $node->title)), 'node/' . $nid, '', t('Apply'), t('Cancel'));
  }
  else {
    $form['intro'] = array(
      '#markup' => t('This job accepts multiple application types, please select one to proceed.'),
    );

    $form['type'] = array(
      '#type' => 'select',
      '#title' => t('Application Type'),
      '#options' => $options,
    );

    $form['#validate'][] = 'drop_jobs_application_form_apply_validate_type';
    $form['#submit'][] = 'drop_jobs_application_form_apply_submit_type';

    return confirm_form($form, t('Apply to job %title?', array('%title' => $node->title)), 'node/' . $nid, '', t('Apply'), t('Cancel'));
  }
}

/**
 * Validation callback.
 *
 * This validates the pre-application form, where the user selects a type.
 *
 * @see drop_jobs_application_form_apply()
 */
function drop_jobs_application_form_apply_validate_type(&$form, &$form_state) {
  // Make sure the user selected a valid type.
  $type = $form_state['values']['type'];
  if (!array_key_exists($type, $form['type']['#options'])) {
    form_set_error($form['type'], t('Invalid application type.'));
  }

  // Make sure the user selected a valid job.
  if (!is_numeric($form_state['values']['nid'])) {
    form_set_error($form_state['values']['nid'], t('Invalid job.'));
  }
}

/**
 * Submit callback.
 *
 * This submits the pre-application form, where the user selects a type.
 *
 * @see drop_jobs_application_form_apply()
 */
function drop_jobs_application_form_apply_submit_type(&$form, &$form_state) {
  // All we do here is just redirect to the type-specific application form.
  $type = $form_state['values']['type'];
  $nid = $form_state['values']['nid'];
  drupal_goto("node/{$nid}/apply/{$type}");
}

/**
 * Validation callback.
 *
 * This validates the final application form, with a type already defined.
 *
 * @see drop_jobs_application_form_apply()
 */
function drop_jobs_application_form_apply_validate_apply($form, &$form_state) {
  // Make sure the user selected a valid job.
  if (!is_numeric($form_state['values']['nid'])) {
    form_set_error($form_state['values']['nid'], t('Invalid job.'));
  }

  // Load application types & filter according to allowed types.
  $allowed = drop_jobs_application_allowed($form_state['values']['nid']);
  $options = array();
  foreach (drop_jobs_application_get_types() as $type_name => $object) {
    if (in_array($type_name, $allowed)) {
      $options[$type_name] = $object->label;
    }
  }

  // Make sure the user selected a valid type.
  $type = $form_state['values']['type'];
  if (!array_key_exists($type, $options)) {
    form_set_error($form['type'], t('Invalid application type.'));
  }
}

/**
 * Submit callback.
 *
 * This submits the final application form, with a type already defined.
 *
 * @see drop_jobs_application_form_apply().
 */
function drop_jobs_application_form_apply_submit_apply($form, &$form_state) {
  $nid = $form_state['values']['nid'];
  $type = $form_state['values']['type'];
  $account = user_uid_optional_load();

  // Prepare values for creating new application.
  $values = array(
    'user' => $account,
    'node' => $nid,
    'type' => $type,
    'status' => DROP_JOBS_APPLICATION_ACTIVE
  );
  $application = drop_jobs_application_create($values);

  // Attach extra submitted values to application.
  entity_form_submit_build_entity('drop_jobs_application', $application, $form, $form_state);

  if (drop_jobs_application_save($application)) {
    // Invoke rules event.
    // This is done here instead of on the entity-level because an application
    // entity can technically be edited and saved after being created.
    if (module_exists('rules')) {
      rules_invoke_event('drop_jobs_application_apply', $application);
    }

    drupal_set_message(t('You have successfully applied to this job!'));
    drupal_goto('node/' . $nid);
  }

  watchdog('drop_jobs_application', 'Error applying to job', array(), WATCHDOG_ERROR);
  drupal_set_message(t('An error ocurred when applying to this job, please contact the site administrators.'), 'error');
}

/**
 * Form callback.
 *
 * Shows a confirmation form for unapplying to a job.
 *
 * @see drop_jobs_application_menu().
 */
function drop_jobs_application_form_unapply($form, $form_state, $nid) {
  $form = array();

  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $nid,
  );

  $options = array();
  foreach (drop_jobs_application_get_types() as $type => $object) {
    $options[$type] = $object->label;
  }

  $account = user_uid_optional_load();
  // Load all user's applications to this job.
  $applications = drop_jobs_application_load_by_user_and_node($account, $nid);
  // Filter active applications.
  $applications = array_filter($applications, 'drop_jobs_application_filter_active');

  // If we only have one application, hide the type select box.
  if (count($applications) === 1) {
    $form['type'] = array(
      '#type' => 'hidden',
      '#value' => reset($applications)->type,
    );
  }
  else {
    $form['type'] = array(
      '#type' => 'select',
      '#title' => t('Application Type'),
      '#options' => $options,
    );
  }

  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm application removal'),
    '#description' => t('Check the above box to confirm your application removal'),
    '#default_value' => FALSE,
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit')
  );

  return $form;
}

/**
 * Validate callback.
 *
 * @see drop_jobs_application_form_unapply().
 */
function drop_jobs_application_form_unapply_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['nid'])) {
    form_set_error($form_state['values']['nid'], t('Invalid job.'));
  }
  if (empty($form_state['values']['type'])) {
    form_set_error($form_state['values']['type'], t('Invalid application type.'));
  }
}

/**
 * Submit callback.
 *
 * @see drop_jobs_application_form_apply().
 */
function drop_jobs_application_form_unapply_submit($form, &$form_state) {
  $nid = $form_state['values']['nid'];
  $type = $form_state['values']['type'];
  $account = user_uid_optional_load();

  $application = drop_jobs_application_load_by_user_and_node($account, $nid, $type);

  if (is_array($application)) {
    // Filter active applications.
    $application = array_filter($application, 'drop_jobs_application_filter_active');
    $application = reset($application);
  }

  $application->status = DROP_JOBS_APPLICATION_INACTIVE;

  if (drop_jobs_application_save($application)) {
    // Invoke rules event.
    if (module_exists('rules')) {
      rules_invoke_event('drop_jobs_application_unapply', $application);
    }

    drupal_set_message(t('You have successfully removed your job application!'));
    drupal_goto('node/' . $nid);
  }
  else {
    // @TODO watchdog?
    drupal_set_message(t('An error ocurred when removing your job application, please contact the site administrators.'), 'error');
  }
}